// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 52799 $
// $Date: 2023-04-06 16:33:55 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTO_CAL_FAIL			D14			// BCR
#define	AUTO_CAL_START			D13			// BCR
#define	AUTO_CAL_MASK			(D12 | D13)	// BCR
#define	INIT_START				D15			// BCR

#define	IRQ_INIT_DONE			0x0000		// BCR
#define	IRQ_AUTO_CAL_DONE		0x0100		// BCR
#define	IRQ_MASK				0x0700		// BCR
#define	IRQ_ACTIVE				0x0800		// BCR



//*****************************************************************************
static void _bcr_mod(dev_data_t* dev, u32 value, u32 mask)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:						arg[0]	= AO12_IOCTL_QUERY_ERROR;	break;
		case AO12_QUERY_AUTO_CAL_MS:	arg[0]	= dev->cache.auto_cal_ms;	break;
		case AO12_QUERY_CHANNEL_MAX:	arg[0]	= dev->cache.channel_max;	break;
		case AO12_QUERY_CHANNEL_QTY:	arg[0]	= dev->cache.channel_qty;	break;
		case AO12_QUERY_COUNT:			arg[0]	= AO12_QUERY_LAST;			break;
		case AO12_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;			break;
		case AO12_QUERY_FIFO_SIZE:		arg[0]	= dev->cache.fifo_size;		break;
		case AO12_QUERY_FREF_ALT_MAX:	arg[0]	= dev->cache.fref_alt_max;	break;
		case AO12_QUERY_FREF_ALT_MIN:	arg[0]	= dev->cache.fref_alt_min;	break;
		case AO12_QUERY_FREF_DEFAULT:	arg[0]	= dev->cache.fref_default;	break;
		case AO12_QUERY_FSAMP_MAX:		arg[0]	= dev->cache.fsamp_max;		break;
		case AO12_QUERY_FSAMP_MIN:		arg[0]	= dev->cache.fsamp_min;		break;
		case AO12_QUERY_INIT_MS:		arg[0]	= dev->cache.initialize_ms;	break;
		case AO12_QUERY_LAST:			arg[0]	= AO12_QUERY_LAST;			break;
		case AO12_QUERY_NCLK_MAX:		arg[0]	= dev->cache.nclk_max;		break;
		case AO12_QUERY_NCLK_MIN:		arg[0]	= dev->cache.nclk_min;		break;
		case AO12_QUERY_NRATE_MAX:		arg[0]	= dev->cache.nrate_max;		break;
		case AO12_QUERY_NRATE_MIN:		arg[0]	= dev->cache.nrate_min;		break;
	}

	return(0);
}



//*****************************************************************************
static int _init_start(dev_data_t* dev, void* arg)
{
	// Initiate initialization.
	os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, INIT_START);
	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	u32			bcr;
	int			i;
	int			mask;
	long		ms			= dev->cache.initialize_ms + 5000;
	long		ms_total	= ms;
	int			ret			= 0;
	os_sem_t	sem;
	int			tmp;
	int			value;
	gsc_wait_t	wt;

	if ((dev->irq.opened) && (gsc_global.driver_unloading == 0))
	{
		ms_total	*= 2;

		// Safely select the Initialize Done interrupt.
		mask	= IRQ_MASK | IRQ_ACTIVE;
		value	= IRQ_INIT_DONE;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= AO12_WAIT_GSC_INIT_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _init_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: INITILIZE DONE IRQ TIMED OUT AFTER %ld ms.\n",
					dev->model,
					(long) ms);
		}
	}

	// Manually wait for completion as the IRQ wait may have ended early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(dev_data_t* dev, void* arg)
{
	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, AUTO_CAL_MASK);
	return(0);
}



//*****************************************************************************
static int _auto_calibrate(dev_data_t* dev, void* arg)
{
	u32			bcr;
	u32			mask;
	long		ms		= dev->cache.auto_cal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Auto-Cal Done interrupt.
	mask	= IRQ_MASK;
	value	= IRQ_AUTO_CAL_DONE;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AO12_WAIT_GSC_AUTO_CAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & AUTO_CAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & AUTO_CAL_FAIL)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_sts(dev_data_t* dev, s32* arg)
{
	u32	reg;
	s32	value;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if ((reg & AUTO_CAL_MASK) == AUTO_CAL_START)
		value	= AO12_AUTO_CAL_STS_ACTIVE;
	else if (reg & AUTO_CAL_FAIL)
		value	= AO12_AUTO_CAL_STS_FAIL;
	else
		value	= AO12_AUTO_CAL_STS_PASS;

	arg[0]	= value;
	return(0);
}



//*****************************************************************************
static int _buffer_clear(dev_data_t* dev, void* arg)
{
	u32	bor;
	int	i;
	int	ret		= 0;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, D11, D11);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		bor	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bor_32);

		if ((bor & D11) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog output buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	// The firmware doesn't support overflow or underflow detection, so there
	// is not corresponding status bit to clear.

	return(ret);
}



//*****************************************************************************
static int _buffer_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_BUFFER_MODE_OPEN,
		AO12_BUFFER_MODE_CIRC,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _buffer_size(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_BUFFER_SIZE_4,
		AO12_BUFFER_SIZE_8,
		AO12_BUFFER_SIZE_16,
		AO12_BUFFER_SIZE_32,
		AO12_BUFFER_SIZE_64,
		AO12_BUFFER_SIZE_128,
		AO12_BUFFER_SIZE_256,
		AO12_BUFFER_SIZE_512,
		AO12_BUFFER_SIZE_1K,
		AO12_BUFFER_SIZE_2K,
		AO12_BUFFER_SIZE_4K,
		AO12_BUFFER_SIZE_8K,
		AO12_BUFFER_SIZE_16K,
		AO12_BUFFER_SIZE_32K,
		AO12_BUFFER_SIZE_64K,
		AO12_BUFFER_SIZE_128K,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 3, 0);
	return(ret);
}



//*****************************************************************************
static int _buffer_status(dev_data_t* dev, u32* arg)
{
	u32	bor;

	bor	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bor_32);

	if (bor & BUFFER_FULL)
		arg[0]	= AO12_BUFFER_STATUS_FULL;
	else if (bor & BUFFER_FULL_3Q)
		arg[0]	= AO12_BUFFER_STATUS_3Q_FULL;
	else if (bor & BUFFER_EMPTY)
		arg[0]	= AO12_BUFFER_STATUS_EMPTY;
	else if (bor & BUFFER_FULL_1Q)
		arg[0]	= AO12_BUFFER_STATUS_1Q_FULL;
	else
		arg[0]	= AO12_BUFFER_STATUS_MEDIUM;

	return(0);
}



//*****************************************************************************
static int _burst_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_BURST_ENABLE_NO,
		AO12_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 0, 0);
	return(ret);
}



//*****************************************************************************
static int _burst_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO12_BURST_READY_NO,
		AO12_BURST_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 1, 1);
	return(ret);
}



//*****************************************************************************
static int _burst_trigger(dev_data_t* dev, void* arg)
{
	_bcr_mod(dev, 0x4, 0x4);
	return(0);
}



//*****************************************************************************
static int _channel_sel(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			0,
			dev->cache.channel_mask,
			dev->vaddr.gsc_csr_32,
			11, 0);
	return(ret);
}



//*****************************************************************************
static int _clock_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_CLOCK_ENABLE_NO,
		AO12_CLOCK_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _clock_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO12_CLOCK_READY_NO,
		AO12_CLOCK_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _clock_ref_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_CLOCK_REF_SRC_PRI,
		AO12_CLOCK_REF_SRC_ALT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_acr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _clock_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_CLOCK_SRC_INT,
		AO12_CLOCK_SRC_EXT_SW,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _clock_sw(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, 0x80, 0x80);
	return(0);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_DATA_FORMAT_2S_COMP,
		AO12_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _ground_sense(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_GROUND_SENSE_DISABLE,
		AO12_GROUND_SENSE_REMOTE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 3, 3);
	return(ret);
}



//*****************************************************************************
static int _irq_sel(dev_data_t* dev, s32* arg)
{
	u32	reg;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		reg		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
		arg[0]	= GSC_FIELD_DECODE(reg, 10, 8);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case AO12_IRQ_INIT_DONE:
			case AO12_IRQ_AUTO_CAL_DONE:
			case AO12_IRQ_BUF_EMPTY:
			case AO12_IRQ_BUF_1Q_FULL:
			case AO12_IRQ_BUF_3Q_FULL:
			case AO12_IRQ_BURST_TRIG_READY:
			case AO12_IRQ_LOAD_READY:
			case AO12_IRQ_LOAD_READY_END:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.
			_bcr_mod(dev, arg[0] << 8, 0xF00);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _load_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO12_LOAD_READY_NO,
		AO12_LOAD_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _load_request(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, 0x200, 0x200);
	return(0);
}



//*****************************************************************************
static int _nclk(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.nclk_min,
			dev->cache.nclk_max,
			dev->vaddr.gsc_acr_32,
			8, 0);
	return(ret);
}



//*****************************************************************************
static int _nrate(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.nrate_min,
			dev->cache.nrate_max,
			dev->vaddr.gsc_srr_32,
			15, 0);
	return(ret);
}



//*****************************************************************************
static int _output_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AO12_OUTPUT_MODE_SEQ,
		AO12_OUTPUT_MODE_SIM,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 7, 7);
	return(ret);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AO12_IO_TIMEOUT_MIN,
			AO12_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AO12_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ AO12_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ AO12_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ AO12_IOCTL_QUERY,				(void*) _query					},
	{ AO12_IOCTL_INITIALIZE,		(void*) initialize_ioctl		},
	{ AO12_IOCTL_AUTO_CALIBRATE,	(void*) _auto_calibrate			},
	{ AO12_IOCTL_AUTO_CAL_STS,		(void*) _auto_cal_sts			},
	{ AO12_IOCTL_BUFFER_CLEAR,		(void*) _buffer_clear			},
	{ AO12_IOCTL_BUFFER_MODE,		(void*) _buffer_mode			},
	{ AO12_IOCTL_BUFFER_SIZE,		(void*) _buffer_size			},
	{ AO12_IOCTL_BUFFER_STATUS,		(void*) _buffer_status			},
	{ AO12_IOCTL_BURST_ENABLE,		(void*) _burst_enable			},
	{ AO12_IOCTL_BURST_READY,		(void*) _burst_ready			},
	{ AO12_IOCTL_BURST_TRIGGER,		(void*) _burst_trigger			},
	{ AO12_IOCTL_CHANNEL_SEL,		(void*) _channel_sel			},
	{ AO12_IOCTL_CLOCK_ENABLE,		(void*) _clock_enable			},
	{ AO12_IOCTL_CLOCK_READY,		(void*) _clock_ready			},
	{ AO12_IOCTL_CLOCK_REF_SRC,		(void*) _clock_ref_src			},
	{ AO12_IOCTL_CLOCK_SRC,			(void*) _clock_src				},
	{ AO12_IOCTL_CLOCK_SW,			(void*) _clock_sw				},
	{ AO12_IOCTL_DATA_FORMAT,		(void*) _data_format			},
	{ AO12_IOCTL_GROUND_SENSE,		(void*) _ground_sense			},
	{ AO12_IOCTL_IRQ_SEL,			(void*) _irq_sel				},
	{ AO12_IOCTL_LOAD_READY,		(void*) _load_ready				},
	{ AO12_IOCTL_LOAD_REQUEST,		(void*) _load_request			},
	{ AO12_IOCTL_NCLK,				(void*) _nclk					},
	{ AO12_IOCTL_NRATE,				(void*) _nrate					},
	{ AO12_IOCTL_OUTPUT_MODE,		(void*) _output_mode			},
	{ AO12_IOCTL_TX_IO_ABORT,		(void*) _tx_io_abort			},
	{ AO12_IOCTL_TX_IO_MODE,		(void*) _tx_io_mode				},
	{ AO12_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ AO12_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ AO12_IOCTL_WAIT_EVENT,		(void*) gsc_wait_event_ioctl	},
	{ AO12_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ -1, NULL }
};



/******************************************************************************
*
*	Function:	buffer_size
*
*	Purpose:
*
*		Return the buffer size in samples.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*		size	The buffer size is returned here.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error code.
*
******************************************************************************/

int buffer_size(dev_data_t* dev, u32* size)
{
	s32	arg	= -1;
	int	ret;

	ret	= _buffer_size(dev, &arg);

	switch (arg)
	{
		default:
		case AO12_BUFFER_SIZE_4:	size[0]	=			4;	break;
		case AO12_BUFFER_SIZE_8:	size[0]	=			8;	break;
		case AO12_BUFFER_SIZE_16:	size[0]	=		   16;	break;
		case AO12_BUFFER_SIZE_32:	size[0]	=		   32;	break;
		case AO12_BUFFER_SIZE_64:	size[0]	=		   64;	break;
		case AO12_BUFFER_SIZE_128:	size[0]	=		  128;	break;
		case AO12_BUFFER_SIZE_256:	size[0]	=		  256;	break;
		case AO12_BUFFER_SIZE_512:	size[0]	=		  512;	break;
		case AO12_BUFFER_SIZE_1K:	size[0]	= 1024L *	1;	break;
		case AO12_BUFFER_SIZE_2K:	size[0]	= 1024L *	2;	break;
		case AO12_BUFFER_SIZE_4K:	size[0]	= 1024L *	4;	break;
		case AO12_BUFFER_SIZE_8K:	size[0]	= 1024L *	8;	break;
		case AO12_BUFFER_SIZE_16K:	size[0]	= 1024L *  16;	break;
		case AO12_BUFFER_SIZE_32K:	size[0]	= 1024L *  32;	break;
		case AO12_BUFFER_SIZE_64K:	size[0]	= 1024L *  64;	break;
		case AO12_BUFFER_SIZE_128K:	size[0]	= 1024L * 128;	break;
	};

	return(ret);
}


